#define PIN_SCL   (11UL)
#define PIN_SDA   (18UL)

// -----------------------------------------------------------------------------
void TWIS_init(
   uint8 const device_address,
   uint8 * const pTX,
   size_t const tx_size,
   uint8 const * const pRX,
   size_t const rx_size)
{
   
   // Disable TWI slave device
   NRF_TWIS1->ENABLE = (TWIS_ENABLE_ENABLE_Disabled << TWIS_ENABLE_ENABLE_Pos);
   
   // Disconnect pins
   NRF_TWIS1->PSEL.SCL = 0xFFFFFFFFUL;
   NRF_TWIS1->PSEL.SDA = 0xFFFFFFFFUL;

   // Disable and clear the TWI slave interrupt
   NVIC_DisableIRQ(SPIM1_SPIS1_TWIM1_TWIS1_SPI1_TWI1_IRQn);
   NVIC_ClearPendingIRQ(SPIM1_SPIS1_TWIM1_TWIS1_SPI1_TWI1_IRQn);

   // Clear all interrupts
   NRF_TWIS1->INTENCLR = 0xFFFFFFFFUL;
   
   // Enable pull-up
   gpGPIO->PIN_CNF[PIN_SCL] = INPUT_PULLUP_SENSENO;
   gpGPIO->PIN_CNF[PIN_SDA] = INPUT_PULLUP_SENSENO;

   // Store the pointers to TX and RX buffer
   gpTX = pTX;
   gpRX = pRX;
   gTX_size = tx_size;

   // Set pins for TWI slave
   NRF_TWIS1->PSEL.SCL  = PIN_SCL;
   NRF_TWIS1->PSEL.SDA  = PIN_SDA;

   // Set slave address 0,1
   NRF_TWIS1->ADDRESS[0] = (uint32)device_address;
   NRF_TWIS1->ADDRESS[1] = (uint32)device_address;
   // Enable address matching for slave address 0
   NRF_TWIS1->CONFIG = (TWIS_CONFIG_ADDRESS0_Enabled << TWIS_CONFIG_ADDRESS0_Pos);
   // Enable address matching for slave address 1
   NRF_TWIS1->CONFIG |= (TWIS_CONFIG_ADDRESS1_Enabled << TWIS_CONFIG_ADDRESS1_Pos);

   // Set the overread character
   NRF_TWIS1->ORC = ORC_TX_CHAR;

   // Set the TX and RX buffers
   // -> TXD.MAXCNT is set every time when an answer is copied into TX buffer
   NRF_TWIS1->TXD.PTR = ptr_to_addr(gpTX);
   NRF_TWIS1->RXD.PTR = ptr_to_addr(gpRX);
   NRF_TWIS1->RXD.MAXCNT = rx_size;
   NRF_TWIS1->TXD.MAXCNT = 0;

   // Clear all possible pending events
   NRF_TWIS1->EVENTS_STOPPED = 0;
   NRF_TWIS1->EVENTS_ERROR = 0;
   NRF_TWIS1->EVENTS_RXSTARTED = 0;
   NRF_TWIS1->EVENTS_TXSTARTED = 0;
   NRF_TWIS1->EVENTS_READ = 0;
   NRF_TWIS1->EVENTS_WRITE = 0;

   // Clear all interrupts
   NRF_TWIS1->INTENCLR = 0xFFFFFFFFUL;

   // Enable interrupt for the STOPPED event
   NRF_TWIS1->INTENSET = (TWIS_INTENSET_STOPPED_Enabled << TWIS_INTENSET_STOPPED_Pos);

   // Enable the IRQ for the TWI slave unit
   NVIC_SetPriority(
      SPIM1_SPIS1_TWIM1_TWIS1_SPI1_TWI1_IRQn,
      (uint32)APP_IRQ_PRIORITY_HIGH);
   NVIC_ClearPendingIRQ(SPIM1_SPIS1_TWIM1_TWIS1_SPI1_TWI1_IRQn);
   NVIC_EnableIRQ(SPIM1_SPIS1_TWIM1_TWIS1_SPI1_TWI1_IRQn);

   // Enable TWI slave device
   NRF_TWIS1->ENABLE = (TWIS_ENABLE_ENABLE_Enabled << TWIS_ENABLE_ENABLE_Pos);
}

